@@ -3,7 +3,7 @@
 #
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2015 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2017 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -21,37 +21,36 @@
 #  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 #  SOFTWARE.
 
-import utils
+from . import utils
 import sys
 import mailbox
 import email
 import email.parser
-import email.Errors
-import cStringIO
-import sgmllib
+import email.errors
+import io
 import glob
 import os
 import re
 import time
-import urllib
+import urllib.request, urllib.parse, urllib.error
 import textwrap
 import pprint
 # SOAP interface to Debian BTS
 import debianbts
 from collections import defaultdict
 
-import checkversions
-from exceptions import (
+from . import checkversions
+from .exceptions import (
     NoNetwork,
     QuertBTSError,
 )
-from urlutils import open_url
+from .urlutils import open_url
 
 
 def msgfactory(fp):
     try:
         return email.message_from_file(fp)
-    except email.Errors.MessageParseError:
+    except email.errors.MessageParseError:
         # Don't return None since that will
         # stop the mailbox iterator
         return ''
@@ -170,31 +169,40 @@ debother = {
     'buildd.debian.org': 'Problems and requests related to the Debian Buildds',
     'buildd.emdebian.org': 'Problems related to building packages for Emdebian',
     'cdimage.debian.org': 'CD Image issues',
-    'cdrom': 'Problems with installation from CD-ROMs',
+    'cdrom': 'Installation system',
+    'cloud.debian.org': 'Issues involving Debian images for public/private clouds',
+    'd-i.debian.org': 'Issues regarding the d-i.debian.org service and general Debian Installer tasks',
     'debian-maintainers': 'Problems and requests related to Debian Maintainers',
-    'debian-policy': 'Proposed changes in the Debian policy documentation',
     'debian-i18n': 'Requests regarding Internationalization (i18n) of the distribution',
+    'debian-live': 'General problems with Debian Live systems',
     'ftp.debian.org': 'Problems with the FTP site and Package removal requests',
-    'general': 'General problems (e.g., that many manpages are mode 755)',
-    'installation-reports': 'Problems with installing Debian',
-    'listarchives': 'Problems with the WWW mailing list archives',
-    'lists.debian.org': 'The mailing lists, debian-*@lists.debian.org.',
-    'mirrors': 'Problems with Debian archive mirrors.',
-    'nm.debian.org': 'New Maintainer process and nm.debian.org website',
+    'general': 'General problems (e.g. "many manpages are mode 755")',
+    'installation-reports': 'Reports of installation problems with stable & testing',
+    'jenkins.debian.org': 'Issues with the jenkins.debian.org service',
+    'lists.debian.org': 'The mailing lists, debian-*@lists.debian.org',
+    'mirrors': 'Problems with the official mirrors',
+    'nm.debian.org': 'New Member process and nm.debian.org webpages',
+    'pet.debian.net': 'The Debian Package Entropy Tracker',
+    'piuparts.debian.org': 'Issues with the piuparts.debian.org service',
     'press': 'Press release issues',
     'project': 'Problems related to project administration',
-    'qa.debian.org': 'Problems related to the quality assurance group',
+    'qa.debian.org': 'The Quality Assurance group',
     'release.debian.org': 'Requests regarding Debian releases and release team tools',
-    'release-notes': 'Problems with the release notes',
+    'release-notes': 'Problems with the Release Notes',
+    'rtc.debian.org': 'Issues in the operation of the Debian RTC services which are not package-specific bugs',
     'security-tracker': 'The Debian Security Bug Tracker',
-    'security.debian.org': 'Problems with the security updates server',
-    'snapshot.debian.org': 'Issues with the snapshot.debian.org service',
+    'security.debian.org': 'The Debian Security Team',
+    'snapshot.debian.org': 'Issues with the snapshot.debian.org service ',
     'spam': 'Spam (reassign spam to here so we can complain about it)',
-    'tech-ctte': 'Issues to be referred to the technical committee',
-    'upgrade-reports': 'Reports of successful and unsucessful upgrades',
+    'sponsorship-requests': 'Requests for package review and sponsorship',
+    'sso.debian.org': 'Problems and requests related to the Debian Single Sign On system',
+    'summit.debconf.org': 'Problems and requests related to the DebConf Summit instance',
+    'tech-ctte': 'The Debian Technical Committee (see the Constitution)',
+    'tracker.debian.org': 'Issues with the Debian Package Tracker and coordination of its maintenance',
+    'upgrade-reports': 'Reports of upgrade problems for stable & testing',
     'wiki.debian.org': 'Problems with the Debian wiki',
     'wnpp': 'Work-Needing and Prospective Packages list',
-    'www.debian.org': 'Problems with the WWW site (including other *.debian.org sites, except alioth)'
+    'www.debian.org': 'Problems with the WWW site'
 }
 
 progenyother = {
@@ -271,12 +279,14 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
             'embedded': "", 'fonts': "", 'games': "", 'gnome': "",
             'gnu-r': "", 'gnustep': "", 'graphics': "", 'hamradio': "",
             'haskell': "", 'httpd': "", 'interpreters': "",
-            'introspection': "", 'java': "", 'kde': "", 'kernel': "",
+            'introspection': "", 'java': "", "javascript": "",
+            'kde': "", 'kernel': "",
             'libdevel': "", 'libs': "", 'lisp': "", 'localization': "",
             'mail': "", 'math': "", 'metapackages': "", 'misc': "",
             'net': "", 'news': "", 'ocaml': "", 'oldlibs': "",
             'otherosfs': "", 'perl': "", 'php': "", 'python': "",
-            'ruby': "", 'science': "", 'shells': "", 'sound': "", 'tex': "",
+            'ruby': "", 'rust': "",
+            'science': "", 'shells': "", 'sound': "", 'tex': "",
             'text': "", 'utils': "", 'vcs': "", 'video': "", 'web': "",
             'x11': "", 'xfce': "", 'zope': "",
         }, 'Choose the section: ', default=section, empty_ok=True)
@@ -387,6 +397,16 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
     archs = None
     version = None
 
+    oldstable = utils.SUITE2CODENAME['oldstable']
+    oldstable_pu = oldstable + '-pu'
+    oldstable_backports = oldstable + '-backports'
+    oldstable_security = oldstable + '-security'
+    stable = utils.SUITE2CODENAME['stable']
+    stable_pu = stable + '-pu'
+    stable_backports = stable + '-backports'
+    stable_security = stable + '-security'
+    testing = utils.SUITE2CODENAME['testing']
+
     tag = ui.menu('What sort of request is this?  (If none of these '
                   'things mean anything to you, or you are trying to report '
                   'a bug in an existing package, please press Enter to '
@@ -395,8 +415,8 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                       'britney': "testing migration script bugs",
                       'transition': "transition tracking",
                       'unblock': "unblock requests",
-                      'wheezy-pu': "wheezy proposed updates requests",
-                      'jessie-pu': "jessie proposed updates requests",
+                      oldstable_pu: "%s proposed updates requests" % oldstable,
+                      stable_pu: "%s proposed updates requests" % stable,
                       'rm': "Stable/Testing removal requests",
                       'other': "None of the other options",
                   }, 'Choose the request type: ', empty_ok=True)
@@ -441,11 +461,11 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
         else:
             package = info[12] or package
 
-    if tag in ('binnmu', 'unblock', 'jessie-pu', 'wheezy-pu', 'rm'):
+    if tag in ('binnmu', 'unblock', stable_pu, oldstable_pu, 'rm'):
         # FIXME: pu/rm should lookup the version elsewhere
         version = info and info[0]
         if online and tag.endswith('-pu'):
-            version = checkversions.get_versions_available(package, timeout, (tag[:-3],)).values()[0]
+            version = list(checkversions.get_versions_available(package, timeout, (tag[:-3],)).values())[0]
         if version:
             cont = ui.select_options(
                 "Latest version seems to be %s, is this the proper one ?" % (version),
@@ -480,13 +500,13 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
     if tag == 'binnmu':
         suite = ui.menu("For which suite are you requesting this binNMU?"
                         "  Don't select anything for \"unstable\"", {
-                            'jessie': "",
-                            'jessie-backports': "",
-                            'jessie-security': "",
-                            'wheezy': "",
-                            'wheezy-backports': "",
-                            'wheezy-security': "",
-                            'stretch': "",
+                            stable: "",
+                            stable_backports: "",
+                            stable_security: "",
+                            oldstable: "",
+                            oldstable_backports: "",
+                            oldstable_security: "",
+                            testing: "",
                             'experimental': "",
                         }, 'Choose the suite: ', empty_ok=True)
         if not suite:
@@ -542,7 +562,7 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
             good = ".depends ~ " + j(listgood)
             bad = ".depends ~ " + j(listbad)
 
-        body += textwrap.dedent(u"""\
+        body += textwrap.dedent("""\
 
                Ben file:
 
@@ -558,7 +578,7 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
         body = ''
     elif tag == 'unblock':
         subject = 'unblock: %s/%s' % (package, version)
-        body = textwrap.dedent(u"""\
+        body = textwrap.dedent("""\
                 Please unblock package %s
 
                 (explain the reason for the unblock here)
@@ -577,7 +597,7 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
     return (subject, severity, headers, pseudos, body, query)
 
 
-itp_template = textwrap.dedent(u"""\
+itp_template = textwrap.dedent("""\
     * Package name    : %(package)s
       Version         : x.y.z
       Upstream Author : Name <somebody@example.org>
@@ -661,7 +681,7 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
 
         if tag == 'ITP':
             headers.append('X-Debbugs-CC: debian-devel@lists.debian.org')
-            pseudos.append(u'Owner: %s' % fromaddr.decode('utf-8', 'replace'))
+            pseudos.append('Owner: %s' % fromaddr)
             ui.log_message('Your report will be carbon-copied to debian-devel, '
                            'per Debian policy.\n')
 
@@ -721,13 +741,13 @@ def dpkg_infofunc():
     multiarch = utils.get_multiarch()
     if debarch:
         if utsmachine == debarch:
-            debinfo = u'Architecture: %s\n' % debarch
+            debinfo = 'Architecture: %s\n' % debarch
         else:
-            debinfo = u'Architecture: %s (%s)\n' % (debarch, utsmachine)
+            debinfo = 'Architecture: %s (%s)\n' % (debarch, utsmachine)
     else:
-        debinfo = u'Architecture: ? (%s)\n' % utsmachine
+        debinfo = 'Architecture: ? (%s)\n' % utsmachine
     if multiarch:
-        debinfo += u'Foreign Architectures: %s\n' % multiarch
+        debinfo += 'Foreign Architectures: %s\n' % multiarch
     debinfo += '\n'
     return debinfo
 
@@ -742,7 +762,7 @@ def ubuntu_infofunc():
 
 def generic_infofunc():
     utsmachine = os.uname()[4]
-    return utils.lsb_release_info() + u'Architecture: %s\n\n' % utsmachine
+    return utils.lsb_release_info() + 'Architecture: %s\n\n' % utsmachine
 
 
 # Supported servers
@@ -858,7 +878,7 @@ def cgi_package_url(system, package, archived=False, source=False,
     if version:
         query['version'] = str(version)
 
-    qstr = urllib.urlencode(query)
+    qstr = urllib.parse.urlencode(query)
     # print qstr
     return '%spkgreport.cgi?%s' % (root, qstr)
 
@@ -867,7 +887,7 @@ def cgi_package_url(system, package, archived=False, source=False,
 def package_url(system, package, mirrors=None, source=False,
                 repeatmerged=True):
     btsroot = get_btsroot(system, mirrors)
-    package = urllib.quote_plus(package.lower())
+    package = urllib.parse.quote_plus(package.lower())
     return btsroot + ('db/pa/l%s.html' % package)
 
 
@@ -908,7 +928,7 @@ def parse_bts_url(url):
 # Dynamically add any additional systems found
 for origin in glob.glob('/etc/dpkg/origins/*'):
     try:
-        fp = file(origin)
+        fp = open(origin)
         system = os.path.basename(origin)
         SYSTEMS[system] = SYSTEMS.get(system, {'otherpkgs': {},
                                                'query-dpkg': True,
@@ -940,198 +960,6 @@ for origin in glob.glob('/etc/dpkg/origins/*'):
         pass
 
 
-# For summary pages, we want to keep:
-#
-# - Contents of <title>...</title>
-# - Contents of <h2>...</h2>
-# - Contents of each <li>
-#
-# For individual bugs, we want to keep:
-# - Contents of <title>...</title>
-# - Contents of every <pre>...</pre> after a <h2>....</h2> tag.
-
-class BTSParser(sgmllib.SGMLParser):
-    def __init__(self, mode='summary', cgi=False, followups=False):
-        import warnings
-        warnings.warn('BTSParse is Deprecated, report a bug if you see this',
-                      DeprecationWarning)
-        sgmllib.SGMLParser.__init__(self)
-        self.hierarchy = []
-        self.lidata = None
-        self.lidatalist = None
-        self.savedata = None
-        self.title = None
-        self.bugcount = 0
-        self.mode = mode
-        self.cgi = cgi
-        self.followups = followups
-        self.inbuglist = self.intrailerinfo = False
-        self.bugtitle = None
-        if followups:
-            self.preblock = []
-        else:
-            self.preblock = ''
-        self.endh2 = False
-
-    # --- Formatter interface, taking care of 'savedata' mode;
-    # shouldn't need to be overridden
-
-    def handle_data(self, data):
-        if self.savedata is not None:
-            self.savedata += data
-
-    # --- Hooks to save data; shouldn't need to be overridden
-
-    def save_bgn(self):
-        self.savedata = ''
-
-    def save_end(self, mode=False):
-        data = self.savedata
-        if not mode and data:
-            data = ' '.join(data.split())
-        self.savedata = None
-        return data
-
-    def start_h1(self, attrs):
-        self.save_bgn()
-        self.oldmode = self.mode
-        self.mode = 'title'
-
-    def end_h1(self):
-        self.title = self.save_end()
-        self.mode = self.oldmode
-
-    def start_h2(self, attrs):
-        if self.lidata:
-            self.check_li()
-
-        self.save_bgn()
-
-    def end_h2(self):
-        if self.mode == 'summary':
-            hiertitle = self.save_end()
-            if 'bug' in hiertitle:
-                self.hierarchy.append((hiertitle, []))
-        self.endh2 = True  # We are at the end of a title, flag <pre>
-
-    def start_ul(self, attrs):
-        if self.mode == 'summary':
-            for k, v in attrs:
-                if k == 'class' and v == 'bugs':
-                    self.inbuglist = True
-
-    def end_ul(self):
-        if self.inbuglist:
-            self.check_li()
-
-        self.inbuglist = False
-
-    def do_br(self, attrs):
-        if self.mode == 'title':
-            self.savedata = ""
-        elif self.mode == 'summary' and self.inbuglist and not self.intrailerinfo:
-            self.bugtitle = self.save_end()
-            self.intrailerinfo = True
-            self.save_bgn()
-
-    def check_li(self):
-        if self.mode == 'summary':
-            if not self.intrailerinfo:
-                self.bugtitle = self.save_end()
-                trailinfo = ''
-            else:
-                trailinfo = self.save_end()
-
-            match = re.search(r'fixed:\s+([\w.+~-]+(\s+[\w.+~:-]+)?)', trailinfo)
-            if match:
-                title = self.bugtitle
-                bits = re.split(r':\s+', title, 1)
-                if len(bits) > 1:
-                    buginfo = '%s [FIXED %s]: %s' % (
-                        bits[0], match.group(1), bits[1])
-                else:
-                    if title.endswith(':'):
-                        title = title[:-1]
-
-                    buginfo = '%s [FIXED %s]' % (title, match.group(1))
-            else:
-                buginfo = self.bugtitle
-
-            self.lidatalist.append(buginfo)
-            self.bugcount += 1
-
-            self.lidata = self.intrailerinfo = False
-
-    def do_li(self, attrs):
-        if self.mode == 'summary' and self.inbuglist:
-            if self.lidata:
-                self.check_li()
-
-            self.lidata = True
-            if self.hierarchy:
-                self.lidatalist = self.hierarchy[-1][1]
-            else:
-                self.lidatalist = []
-            self.save_bgn()
-
-    def start_pre(self, attrs):
-        "Save <pre> when we follow a </h2>"
-        if self.followups:
-            if not self.endh2:
-                return
-        else:
-            if self.cgi and self.preblock:
-                return
-
-        self.save_bgn()
-
-    def end_pre(self):
-        if self.followups:
-            if not self.endh2:
-                return
-            self.endh2 = False  # Done with a report, reset </h2>.
-            stuff = self.save_end(1)
-            if not self.cgi:
-                self.preblock.insert(0, stuff)
-            else:
-                self.preblock.append(stuff)
-        elif not (self.preblock and self.cgi):
-            self.preblock = self.save_end(1)
-
-    def reorganize(self):
-        if not self.hierarchy:
-            return
-
-        newhierarchy = []
-        fixed = []
-        fixedfinder = re.compile(r'\[FIXED ([^\]]+)\]')
-        resolvedfinder = re.compile(r'Resolved')
-
-        for (title, buglist) in self.hierarchy:
-            if 'Resolved' in title:
-                newhierarchy.append((title, buglist))
-                continue
-
-            bugs = []
-            for bug in buglist:
-                if fixedfinder.search(bug):
-                    fixed.append(bug)
-                else:
-                    bugs.append(bug)
-
-            if bugs:
-                title = ' '.join(title.split()[:-2])
-                if len(bugs) != 1:
-                    title += ' (%d bugs)' % len(bugs)
-                else:
-                    title += ' (1 bug)'
-
-                newhierarchy.append((title, bugs))
-
-        if fixed:
-            self.hierarchy = [('Bugs fixed in subsequent releases (%d bugs)' % len(fixed), fixed)] + newhierarchy
-
-
 # TODO: to be removed
 def parse_html_report(number, url, http_proxy, timeout, followups=False, cgi=True):
     page = open_url(url, http_proxy, timeout)
@@ -1183,7 +1011,7 @@ def parse_mbox_report(number, url, http_proxy, timeout, followups=False):
         return None
 
     # Make this seekable
-    wholefile = cStringIO.StringIO(page.read())
+    wholefile = io.StringIO(page.read())
 
     try:
         page.fp._sock.recv = None
@@ -1234,60 +1062,6 @@ def parse_mbox_report(number, url, http_proxy, timeout, followups=False):
     return (title, output)
 
 
-def get_cgi_reports(package, timeout, system='debian', http_proxy='',
-                    archived=False, source=False, version=None):
-    try:
-        page = open_url(cgi_package_url(system, package, archived, source,
-                                        version=version), http_proxy, timeout)
-    except:
-        raise NoNetwork
-
-    if not page:
-        return (0, None, None)
-
-    # content = page.read()
-    # if 'Maintainer' not in content:
-    #    return (0, None, None)
-
-    parser = BTSParser(cgi=True)
-    for line in page:
-        try:
-            line = line.decode('utf-8')  # BTS pages are encoded in utf-8
-        except UnicodeDecodeError:
-            # page has a bad char
-            line = line.decode('utf-8', 'replace')
-        parser.feed(line)
-    parser.close()
-    try:
-        page.fp._sock.recv = None
-    except:
-        pass
-    page.close()
-
-    # Reorganize hierarchy to put recently-fixed bugs at top
-    parser.reorganize()
-
-    # Morph @ 2008-08-15; due to BTS output format changes
-    try:
-        parser.hierarchy.remove(('Select bugs', []))
-    except:
-        pass
-
-    data = (parser.bugcount, parser.title, parser.hierarchy)
-    del parser
-
-    return data
-
-
-def get_cgi_report(number, timeout, system='debian', http_proxy='', archived=False,
-                   followups=False):
-    number = int(number)
-
-    url = cgi_report_url(system, number, archived='no', mbox=True)
-    return parse_mbox_report(number, url, http_proxy, timeout, followups)
-    # return parse_html_report(number, url, http_proxy, followups, cgi=True)
-
-
 def get_btsroot(system, mirrors=None):
     if mirrors:
         alternates = SYSTEMS[system].get('mirrors')
@@ -1300,14 +1074,14 @@ def get_btsroot(system, mirrors=None):
 def get_reports(package, timeout, system='debian', mirrors=None, version=None,
                 http_proxy='', archived=False, source=False):
     if system == 'debian':
-        if isinstance(package, basestring):
+        if isinstance(package, str):
             if source:
                 pkg_filter = 'src'
             else:
                 pkg_filter = 'package'
             bugs = debianbts.get_bugs(pkg_filter, package)
         else:
-            bugs = map(int, package)
+            bugs = list(map(int, package))
 
         try:
             # retrieve bugs and generate the hierarchy
@@ -1334,7 +1108,7 @@ def get_reports(package, timeout, system='debian', mirrors=None, version=None,
         return (len(bugs), 'Bug reports for %s' % package, hier)
 
     # XXX: is the code below used at all now? can we remove it?
-    if isinstance(package, basestring):
+    if isinstance(package, str):
         if SYSTEMS[system].get('cgiroot'):
             try:
                 result = get_cgi_reports(package, timeout, system, http_proxy, archived,
